//Node.cpp
#include "Node.h"
#include <assert.h>
#include <iostream>
#include <stdio.h>

void _split(const std::string line, const std::string separator, std::vector<std::string>& tokens) 
{
    size_t seplen = separator.length();
    size_t start = 0;
    size_t end = line.length();
 
    while ((start >= 0) && (start < end)) 
	{
        start = start + seplen;
        size_t stop = line.find(separator, start);
        if ((stop < 0) || (stop > end)) 
			stop = end;
        tokens.push_back(start==seplen ? line.substr(0, stop)  : line.substr(start, stop-start));
        start = stop;
    }
}
//-----------------------------------------------------------------------------
Node::Node()
: m_pParent( NULL ) 
, m_child( NULL )
, m_pData( NULL )
{	
}
//-----------------------------------------------------------------------------
Node::~Node()
{	
	if( m_pData )
	{
		delete m_pData;
		m_pData = NULL;
	}

	//Ӽ 
	{
		AttributeList::iterator iter = m_attribute.begin();
		AttributeList::iterator endIter = m_attribute.end();
		for( ; iter != endIter; ++iter )
		{
			delete (*iter);
		}
		m_attribute.clear();
	}
	//ڽ 
	{
		ChildList::iterator iter = m_child.begin();
		ChildList::iterator endIter = m_child.end();
		for( ; iter != endIter; ++iter )
		{
			delete (*iter);
		}
		m_child.clear();

	}
}
//-----------------------------------------------------------------------------
void Node::AttachChild( Node* pChild )
{
	pChild->m_pParent = this;
	m_child.push_back( pChild );
}
//-----------------------------------------------------------------------------
void Node::DetachChild( Node* pChild )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		if( pChild == *iter )
		{
			delete (*iter);
			m_child.erase( iter );
			break;
		}
	}
}
//-----------------------------------------------------------------------------
Node* Node::GetParent()
{
	return m_pParent;
}
//-----------------------------------------------------------------------------
Node* Node::GetRoot()
{
	return m_pParent ? m_pParent->GetRoot() : this;
}
//---------------------------------------------------------------------------
void Node::SetData( std::string* pData )
{
	m_pData = pData;
}
//-----------------------------------------------------------------------------
std::string* Node::GetData()
{
	return m_pData;
}
//-----------------------------------------------------------------------------
std::string* Node::GetClassName()
{
	return &m_className;
}
//-----------------------------------------------------------------------------
void Node::SetClassName( const char* szClassName )
{
	m_className = szClassName;
}
//-----------------------------------------------------------------------------
void Node::AddAttribute( const char* pName, const char* pValue )
{
	m_attribute.push_back(  new Attribute( pName, pValue ) );
}
//-----------------------------------------------------------------------------
void Node::FindNodeAll( const std::vector<std::string>& tok, int iDepth, NodeVector& nodeArr )
{
	if( tok.size() <= (unsigned)iDepth )
	{
		nodeArr.push_back( this );
		return;
	}

	const char* pName = tok[iDepth].c_str();
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		Node* pChild = *iter;
		if( *pChild->GetClassName() == pName )
		{
			 pChild->FindNodeAll( tok, iDepth + 1, nodeArr );
		}
	}

	return;
}
//-----------------------------------------------------------------------------
void Node::FindNode( const char* pName, NodeVector& nodeArr )
{
	std::vector<std::string> tok;
	_split( pName, "/", tok );

	FindNodeAll( tok, 0, nodeArr );
}
//-----------------------------------------------------------------------------
void Node::PrintAll( Node* pNode )
{
	std::cout << "Node Name : " << pNode->GetParent() << " : " << pNode << " : " 
		<< pNode->GetClassName()->c_str() << std::endl;
	if( pNode->m_child.empty() == true )
		return;
	
	ChildList::iterator iter = pNode->m_child.begin();
	ChildList::iterator endIter = pNode->m_child.end();
	for( ; iter != endIter; ++iter )
	{
		PrintAll( *iter );
	}
}
